@@ -7,7 +7,7 @@ import subprocess
 import threading
 import time
 import ConfigParser
-from gi.repository import Gtk, GObject
+from gi.repository import GLib, GObject, Gtk
 # Import special features if we're running Ubuntu Unity
 if (os.environ.get('DESKTOP_SESSION') == 'ubuntu' or
         os.environ.get('DESKTOP_SESSION') == 'ubuntu-2d'):
@@ -36,7 +36,8 @@ options = {
     'program': 'rdesktop',
     'homeshare': 'false',
     'grabkeyboard': 'false',
-    'fullscreen': 'false'
+    'fullscreen': 'false',
+    'clioptions': ''
 }
 
 
@@ -55,13 +56,8 @@ def save_config(section, window=None):
     if not config.has_section(section):
         config.add_section(section)
     # Set all selected options
-    config.set(section, 'host', options['host'])
-    config.set(section, 'user', options['user'])
-    config.set(section, 'geometry', options['geometry'])
-    config.set(section, 'program', options['program'])
-    config.set(section, 'homeshare', options['homeshare'])
-    config.set(section, 'grabkeyboard', options['grabkeyboard'])
-    config.set(section, 'fullscreen', options['fullscreen'])
+    for opt in options:
+        config.set(section, opt, options[opt])
     write_config()
 
 
@@ -74,13 +70,9 @@ def delete_config(section):
 # Set options based on section in config file
 def read_config(section):
     if os.path.exists(configfile):
-        options['host'] = config.get(section, 'host')
-        options['user'] = config.get(section, 'user')
-        options['geometry'] = config.get(section, 'geometry')
-        options['program'] = config.get(section, 'program')
-        options['homeshare'] = config.get(section, 'homeshare')
-        options['grabkeyboard'] = config.get(section, 'grabkeyboard')
-        options['fullscreen'] = config.get(section, 'fullscreen')
+        for opt in options:
+            if config.has_option(section, opt):
+                options[opt] = config.get(section, opt)
 
 
 # Make a list of all profiles in config file.  Sort the order alphabetically,
@@ -145,6 +137,8 @@ def run_program(window):
         params.append(client_opts[client]['grabkeyboard'])
     if options['homeshare'] == 'true':
         params.append(client_opts[client]['homeshare'])
+    if options['clioptions'] != '':
+        params.append(options['clioptions'])
     # Hostname goes last in the list of parameters
     params.append(client_opts[client]['host']
                   + '%s' % str.strip(options['host']))
@@ -166,7 +160,7 @@ class WorkerThread(threading.Thread):
         global p
         p = subprocess.Popen(self.cmdline, stderr=subprocess.PIPE)
         time.sleep(2)
-        GObject.idle_add(self.callback)
+        GLib.idle_add(self.callback)
 
 
 # GUI stuff
@@ -196,7 +190,7 @@ class MainWindow(Gtk.Window):
         """
 
         # Menu bar
-        action_group = Gtk.ActionGroup("Menu")
+        action_group = Gtk.ActionGroup(name="Menu")
         self.add_file_menu_actions(action_group)
         self.add_help_menu_actions(action_group)
         uimanager = self.create_ui_manager()
@@ -209,11 +203,12 @@ class MainWindow(Gtk.Window):
         self.add(grid)
 
         # Labels for text entry fields and comboboxes
-        profileslabel = Gtk.Label("Profile")
-        hostlabel = Gtk.Label("Host")
-        userlabel = Gtk.Label("Username")
-        geometrylabel = Gtk.Label("Geometry")
-        programlabel = Gtk.Label("RDP Client")
+        profileslabel = Gtk.Label(label="Profile")
+        hostlabel = Gtk.Label(label="Host")
+        userlabel = Gtk.Label(label="Username")
+        geometrylabel = Gtk.Label(label="Geometry")
+        clioptionslabel = Gtk.Label(label="CLI Options")
+        programlabel = Gtk.Label(label="RDP Client")
 
         # Profiles combobox
         self.profiles_combo = Gtk.ComboBoxText.new_with_entry()
@@ -242,6 +237,10 @@ Can be set to a specific resolution or a percentage:
 e.g. "1024x768" or "80%"''')
         self.geometryentry.connect("activate",
                                    self.enter_connect, self.geometryentry)
+        self.clioptionsentry = Gtk.Entry()
+        self.clioptionsentry.set_tooltip_text('''Extra CLI options''')
+        self.clioptionsentry.connect("activate",
+                                   self.enter_connect, self.clioptionsentry)
 
         # Combobox for program selection
         program_store = Gtk.ListStore(str)
@@ -260,21 +259,21 @@ e.g. "1024x768" or "80%"''')
         self.program_combo.add_attribute(self.program_renderer_text, "text", 0)
 
         # Checkbox for sharing our home directory
-        self.homedirbutton = Gtk.CheckButton("Share Home Dir")
+        self.homedirbutton = Gtk.CheckButton(label="Share Home Dir")
         self.homedirbutton.set_tooltip_text('Share local home directory with '
                                             'RDP server')
         self.homedirbutton.connect("toggled", self.on_button_toggled,
                                    "homeshare")
 
         # Checkbox for grabbing the keyboard
-        self.grabkeyboardbutton = Gtk.CheckButton("Grab Keyboard")
+        self.grabkeyboardbutton = Gtk.CheckButton(label="Grab Keyboard")
         self.grabkeyboardbutton.set_tooltip_text('Send all keyboard inputs to '
                                                  'RDP server')
         self.grabkeyboardbutton.connect("toggled", self.on_button_toggled,
                                         "grabkeyboard")
 
         # Checkbox for fullscreen view
-        self.fullscreenbutton = Gtk.CheckButton("Fullscreen")
+        self.fullscreenbutton = Gtk.CheckButton(label="Fullscreen")
         self.fullscreenbutton.set_tooltip_text('Run RDP client in fullscreen '
                                                'mode')
         self.fullscreenbutton.connect("toggled", self.on_button_toggled,
@@ -297,7 +296,8 @@ e.g. "1024x768" or "80%"''')
         grid.attach(hostlabel, 0, 8, 4, 4)
         grid.attach(userlabel, 0, 12, 4, 4)
         grid.attach(geometrylabel, 0, 16, 4, 4)
-        grid.attach(programlabel, 0, 20, 4, 4)
+        grid.attach(clioptionslabel, 0, 20, 4, 4)
+        grid.attach(programlabel, 0, 24, 4, 4)
         grid.attach_next_to(self.profiles_combo, profileslabel,
                             Gtk.PositionType.RIGHT, 8, 4)
         grid.attach_next_to(self.hostentry, hostlabel,
@@ -306,14 +306,16 @@ e.g. "1024x768" or "80%"''')
                             Gtk.PositionType.RIGHT, 8, 4)
         grid.attach_next_to(self.geometryentry, geometrylabel,
                             Gtk.PositionType.RIGHT, 8, 4)
+        grid.attach_next_to(self.clioptionsentry, clioptionslabel,
+                            Gtk.PositionType.RIGHT, 8, 4)
         grid.attach_next_to(self.program_combo, programlabel,
                             Gtk.PositionType.RIGHT, 8, 4)
-        grid.attach(self.homedirbutton, 0, 24, 4, 4)
+        grid.attach(self.homedirbutton, 0, 28, 4, 4)
         grid.attach_next_to(self.grabkeyboardbutton, self.homedirbutton,
                             Gtk.PositionType.RIGHT, 4, 4)
         grid.attach_next_to(self.fullscreenbutton, self.grabkeyboardbutton,
                             Gtk.PositionType.RIGHT, 4, 4)
-        grid.attach(quitbutton, 0, 28, 4, 4)
+        grid.attach(quitbutton, 0, 32, 4, 4)
         grid.attach_next_to(self.connectbutton, quitbutton,
                             Gtk.PositionType.RIGHT, 8, 4)
         grid.attach_next_to(self.spinner, quitbutton,
@@ -441,7 +443,7 @@ e.g. "1024x768" or "80%"''')
 
     # Triggered when the file menu is used
     def add_file_menu_actions(self, action_group):
-        action_filemenu = Gtk.Action("FileMenu", "File", None, None)
+        action_filemenu = Gtk.Action(name="FileMenu", label="File", tooltip=None, stock_id=None)
         action_group.add_action(action_filemenu)
         # Why do the functions here execute on startup if we add parameters?
         action_group.add_actions([("SaveCurrentConfig", None,
@@ -453,7 +455,7 @@ e.g. "1024x768" or "80%"''')
         action_group.add_actions([("DeleteCurrentConfig", None,
                                    "Delete Current Profile", None, None,
                                    self.delete_current_config)])
-        action_filequit = Gtk.Action("FileQuit", None, None, Gtk.STOCK_QUIT)
+        action_filequit = Gtk.Action(name="FileQuit", label=None, tooltip=None, stock_id=Gtk.STOCK_QUIT)
         action_filequit.connect("activate", self.quit)
         action_group.add_action(action_filequit)
 
@@ -527,11 +529,12 @@ e.g. "1024x768" or "80%"''')
         options['host'] = self.hostentry.get_text()
         options['user'] = self.userentry.get_text()
         options['geometry'] = self.geometryentry.get_text()
+        options['clioptions'] = self.clioptionsentry.get_text()
 
     # Generic warning dialog
     def on_warn(self, widget, title, message):
-        dialog = Gtk.MessageDialog(self, 0, Gtk.MessageType.WARNING,
-                                   Gtk.ButtonsType.OK, title,
+        dialog = Gtk.MessageDialog(transient_for=self, flags=0, message_type=Gtk.MessageType.WARNING,
+                                   buttons=Gtk.ButtonsType.OK, text=title,
                                    title='Rocket Depot')
         dialog.format_secondary_text(message)
         dialog.run()
@@ -541,7 +544,7 @@ e.g. "1024x768" or "80%"''')
     def on_about(self, widget):
         about = Gtk.AboutDialog()
         about.set_program_name("Rocket Depot")
-        about.set_version("0.15")
+        about.set_version("0.16")
         about.set_copyright("2014 David Roble")
         about.set_comments("rdesktop/xfreerdp Frontend")
         about.set_website("https://github.com/robled/rocket-depot")
@@ -553,6 +556,7 @@ e.g. "1024x768" or "80%"''')
         self.hostentry.set_text(options['host'])
         self.userentry.set_text(options['user'])
         self.geometryentry.set_text(options['geometry'])
+        self.clioptionsentry.set_text(options['clioptions'])
         self.program_combo.set_active(self.programs[options['program']])
         if options['homeshare'] == 'true':
             self.homedirbutton.set_active(True)
@@ -569,8 +573,6 @@ e.g. "1024x768" or "80%"''')
 
 
 def _main():
-    # Begin GUI thread stuff early
-    GObject.threads_init()
     # Local user homedir and config file
     global homedir
     global configfile
